from time import time
from datetime import timedelta
from discord.ext import commands

from templatebot import Bot


class Default(commands.Cog):
    """A general purpose cog for tasks like cog loading, reloading, and unloading."""

    def __init__(self, bot: Bot):
        self.bot = bot
        self.ltime = time()

    @commands.group(name="cogs")
    @commands.is_owner()
    async def cogs_group(self, ctx: commands.Context):
        """Perform actions such as reloading cogs"""
        if ctx.invoked_subcommand is None:
            await ctx.send(f"Usage: `!cogs <load | reload | unload> [list of cogs]`")

    @cogs_group.command(name="load")
    async def load_cogs(self, ctx: commands.Context, *cognames):
        """Load a set of cogs"""
        log = ""
        for cog in cognames:
            cog = "bot.cogs." + cog
            try:
                self.bot.load_extension(cog)
                log += f"Successfully loaded cog {cog}\n"
            except Exception as e:
                log += f"Failed to load cog {cog}: {e}\n"
                self.bot.logger.error(f"Cog loading: failed to load {cog}: {e}")

        self.bot.logger.info(f"Loaded cog(s):\n{log}")
        await ctx.send(log)

    @cogs_group.command(name="reload")
    async def reload_cogs(self, ctx: commands.Context, *cognames):
        """Reload a set of cogs"""
        log = ""
        for cog in cognames:
            cog = "bot.cogs." + cog
            try:
                self.bot.reload_extension(cog)
                log += f"Successfully reloaded cog {cog}\n"
            except Exception as e:
                log += f"Failed to reload cog {cog}: {e}\n"
                self.bot.logger.error(f"Cog reloading: failed to reload {cog}: {e}")

        self.bot.logger.info(f"Reloaded cog(s):\n{log}")
        await ctx.send(log)

    @cogs_group.command(name="unload")
    async def unload_cogs(self, ctx: commands.Context, *cognames):
        """Unload a set of cogs"""
        log = ""
        for cog in cognames:
            cog = "bot.cogs." + cog
            try:
                self.bot.unload_extension(cog)
                log += f"Successfully unloaded cog {cog}\n"
            except Exception as e:
                log += f"Failed to unload cog {cog}: {e}\n"
                self.bot.logger.error(f"Cog unloading: failed to unload {cog}: {e}")

        self.bot.logger.info(f"Unloaded cog(s):\n{log}")
        await ctx.send(log)

    @commands.command(name="restart", aliases=["reboot", "shutdown"])
    @commands.is_owner()
    async def restart(self, ctx: commands.Context):
        """Make the bot logout"""
        await ctx.send("Restarting...")
        self.bot.logger.info(f"Shutting down {self.bot.name}")
        await self.bot.close()

    @commands.command(name="ping")
    @commands.is_owner()
    async def ping(self, ctx: commands.Context):
        t_start = time.time()
        m = await ctx.channel.send("Testing RTT for message editing.")
        await m.edit(content="Testing...")
        rtt = time.time() - t_start
        await m.edit(
            content=f"Pong!\nMessage edit RTT: {round(rtt * 1000, 2)}ms\nWebsocket Latency: {round(self.bot.latency * 1000, 2)}ms")

    @commands.command(name="uptime")
    @commands.is_owner()
    async def uptime(self, ctx: commands.Context):
        bot_uptime = timedelta(seconds=round(time() - self.bot.ltime))
        cog_uptime = timedelta(seconds=round(time() - self.ltime))
        await ctx.send(f"Bot uptime: {bot_uptime}\nCog load uptime: {cog_uptime}")

    @commands.Cog.listener()
    async def on_ready(self):
        await self.bot.logger.info(f"{self.bot.name} has received READY event, logged in as {self.bot.user} ({self.bot.user.id})")


def setup(bot: Bot):
    bot.add_cog(Default(bot))



import asyncio
from aiohttp import ClientSession
from requests import post

from datetime import datetime
from termcolor import colored

termc = {
    "debug": 'white',
    "info": 'green',
    "warn": 'yellow',
    "error": 'red',
    "critical": 'red'
}


class LogLevel:
    DEBUG = 5
    INFO = 4
    WARNING = 3
    ERROR = 2
    CRITICAL = 1


class WebhookLogger:
    def __init__(self, name: str, url: str = None, level: int = LogLevel.INFO, loop=asyncio.get_event_loop()):
        self.name = name
        self.url = url
        self.level = level
        self.sess = ClientSession()
        self.loop = loop

    def create_content(self, logtype: str, content: str):
        timestamp = str(datetime.now()).split(".")[0]
        content = f"__**{logtype}**__ @ {timestamp}:\n{content}"
        data = {
            "username": self.name,
            "avatar_url": "https://vcokltfre.github.io/static/img/service.png",
            "content": content,
            "allowed_mentions": {
                "users": False,
                "roles": False,
                "everyone": False
            }
        }
        return data

    def send(self, logtype: str, content: str):
        print(colored(f"[{logtype}] {content}", termc[logtype.lower()]))
        if not self.url: return
        data = self.create_content(logtype, content)
        post(self.url, json=data)

    async def send_async(self, logtype: str, content: str):
        print(colored(f"[{logtype}] {content}", termc[logtype.lower()]))
        if not self.url: return
        data = self.create_content(logtype, content)
        if self.sess.closed:
            self.sess = ClientSession()

        task = self.loop.create_task(self.sess.post(self.url, json=data))
        await task

    ## Synchronous methods

    def debug_s(self, content):
        self.send("debug", content)

    def info_s(self, content):
        self.send("info", content)

    def warn_s(self, content):
        self.send("warn", content)

    def error_s(self, content):
        self.send("error", content)

    def critical_s(self, content):
        self.send("critical", content)

    ## Asynchronous methods

    async def debug(self, content):
        await self.send_async("debug", content)

    async def info(self, content):
        await self.send_async("info", content)

    async def warn(self, content):
        await self.send_async("warn", content)

    async def error(self, content):
        await self.send_async("error", content)

    async def critical(self, content):
        await self.send_async("critical", content)



from setuptools import setup, find_packages

version = "1.0.2"

setup(
	name='templatebot',
	version=version,
	packages=find_packages(),
	url='https://github.com/vcokltfre/template-bot-v2',
	license='MIT',
	author='vcokltfre',
	long_description=open("README.md").read(),
	long_description_content_type="text/markdown",
	install_requires=[open("requirements.txt").read().split("\n")],
	description='A template Discord bot',
	python_requires='>=3.6',
)


from time import time
from discord.ext import commands
from typing import List

from .utils.logger import WebhookLogger, LogLevel


class Bot(commands.Bot):
    """A subclassed version of commands.Bot"""

    def __init__(self, name: str, command_prefix, logging_url: str = None, loglevel: int = LogLevel.INFO, *args, **kwargs):
        self.name = name
        self.ltime = time()
        super().__init__(command_prefix=command_prefix, *args, **kwargs)

        self.logger = WebhookLogger(name=name, url=logging_url, level=loglevel)

    def load_initial_cogs(self, *cogs):
        """Loads the initial cogs"""

        success, failed = 0, 0

        cogs = set(cogs)
        cogs.add("templatebot.cogs.default")
        cogs.add("jishaku")

        for cog in cogs:
            try:
                super().load_extension(cog)
                success += 1
            except Exception as e:
                self.logger.error_s(f"Cog {cog} experienced an error during loading: {e}")
                failed += 1

        self.logger.info_s(f"Cog loading complete! (Total: {success + failed} | Loaded: {success} | Failed: {failed})")
